#include <cstdint>
#include <iostream>
#include <fstream>
#include <sstream>
#include <vector>
using namespace std;

void internal(const string &message)
{
    cout << "Internal error\n"
         << "Prosím obráťte sa na organizátorov. Detaily chyby:\n"
         << message << endl;
    exit(2);
}

void zle(const string &message)
{
    cout << "Zlá odpoveď: " << message << endl;
    exit(1);
}

void ok(const string &message = "")
{
    cout << "OK";
    if (message != "") cout << " (" << message << ")";
    cout << endl;
    exit(0);
}

int main(int argc, char* argv[]) {
    if (argc < 5) {
        cout << "Usage: checker <testcase_id> <input_file> <target_output> <their_output>\n";
        return 2;
    }

    int testcase_id = atoi(argv[1]);

    int usedWeights[10234] = {};

    ifstream f_inp(argv[2]), f_our(argv[3]), f_their(argv[4]);

    uint n, v, startingStateSize;
    f_inp >> n >> v >> startingStateSize;

    vector<uint> states[2];
    for (uint i = 0; i < startingStateSize; i++) {
        uint w;
        f_inp >> w;
        states[0].push_back(w);
        usedWeights[w] += 2;
    }

    states[1] = states[0];

    int ourNumOps = 0;
    f_our >> ourNumOps;

    uint ourCost = 0;
    if (ourNumOps != -1) {
        f_our >> ourCost;
    }

    int numOps = 0;
    if (!(f_their >> numOps)) {
        zle("Nepodarilo sa prečítať l (počet operácii).");
    }

    if (numOps == -1) {
        if (ourNumOps != -1) zle("Riešenie existuje, ale súťažiaci tvrdí, že nie.");
        ok();
    }
    
    uint claimedCost = 0;
    if (!(f_their >> claimedCost)) {
        zle("Nepodarilo sa prečítať m (celkovú hmotnosť presunov).");
    }

    /*
    Vo vstupoch 5 a 6 stačí mať správne m.
    */

    if (testcase_id == 5 || testcase_id == 6) {
        if (ourCost == claimedCost) ok("m je správne, zvyšok nekontrolujem");
        zle("Vypísaná hodnota m sa líši od optimálnej.");
    }
    
    /*
    Vo vstupoch 3 a 4 netreba optimalizovať hmotnosť presunutých kotúčov.
    */

    if (numOps < 0) zle("Hodnota l nesmie byť záporná.");
    if (testcase_id == 3 || testcase_id == 4) {
        if (numOps > 1000) zle("Hodnota l nesmie byť väčšia ako 1000.");
    } else {
        if (numOps > 1000) zle("Hodnota l väčšia ako 1000 určite nezodpovedá optimálnemu riešeniu.");
    }

    string line1;
    getline(f_their, line1); // newline za ich m

    uint totalCost = 0;
    for (int oi = 0; oi < numOps; oi++) {
        string line;
        if (!getline(f_their, line)) {
            zle("Primálo riadkov s inštrukciami.");
        }

        stringstream sstream(line);

        string side, opType;
        if (!(sstream >> side >> opType)) {
            zle("Zlý formát operácie: " + line);
        }

        if (side != "L" && side != "R") {
            zle("Zlý formát operácie: " + line);
        }

        vector<uint>& state = states[(side == "L" ? 0 : 1)];

        if (opType == "-") {
            if (state.empty()) {
                zle("Nemožno odobrať z konca tyče, nič tam nie je.");
            } else {
                usedWeights[state.back()]--;
                totalCost += state.back();
                state.pop_back();
            }
        }
        else if (opType == "+") {
            uint w;
            if (!(sstream >> w)) {
                zle("Zlý formát operácie: " + line);
            }
            if (w > n || w == 0) {
                zle("Neexistujúca hmotnosť kotúča.");
            }
            if (usedWeights[w] == 2) {
                zle("Chce pridať kotúč ktorého oba kusy sú už na činke.");
            }
            state.push_back(w);
            totalCost += w;
            usedWeights[w]++;
        }
        else {
            zle("Zlý formát operácie: " + line);
        }
    }

    if (totalCost != claimedCost) {
        stringstream tmp;
        tmp << "Vypísané m bolo " << claimedCost << " ale vypísané operácie majú súčet hmotností " << totalCost;
        zle(tmp.str());
    }

    if (states[0].size() != states[1].size()) {
        zle("Výsledná činka nie je symetrická.");
    }

    for (uint i = 0; i < states[0].size(); i++) {
        if (states[0][i] != states[1][i]) {
            zle("Výsledná činka nie je symetrická.");
        }
    }

    for (uint i = 0; i + 1 < states[0].size(); i++) {
        if (states[0][i] <= states[0][i+1]) {
            zle("Hmotnosti kotúčov vo výslednej činke musia od stredu ku kraju klesať.");
        }
    }

    uint totalWeight = 20;
    for (uint i = 0; i < states[0].size(); i++) {
        totalWeight += 2 * states[0][i];
    }
    
    if (totalWeight != v) {
        zle("Hmotnosť výslednej činky nie je rovná zadanému v.");
    }

    if (ourNumOps == -1) {
        internal("Našli ste riešenie, ale organizátori si myslia, že riešenie neexistuje.");
    }
    if (totalCost < ourCost && ourCost != UINT32_MAX) {
        internal("Našli ste lepšie riešenie ako organizátori.");
    }
        
    if (getline(f_their, line1)) {
        zle("Priveľa riadkov s inštrukciami.");
    }

    if (testcase_id == 3 || testcase_id == 4) ok("správne postavená činka, sedí v, nekontrolujem m");

    if (totalCost > ourCost) {
        zle("Dosiahnutá hodnota m nie je optimálna.");
    }

    ok();

    return 0;
}
